﻿--[[
Name: LibAvion-2.0
Revision: $REV$
Author(s): Sirow (EU-Gorgonash)
Documentation: http://www.wowace.com/wiki/LibAvion-2.0
SVN: http://svn.wowace.com/wowace/trunk/LibAvion-2.0/
Description: Allows to trigger visual effects (Library).
Dependencies: LibSimpleTimer-1.0
]]


-------------------------------------------------------------------------------
-- Library - Version Info & Dependencies
-------------------------------------------------------------------------------
-- Version
local MAJOR 		= "LibAvion-2.0"
local MAJOR_EMBED	= "LibAvionEmbed-2.0"
local MINOR 		= tonumber(("$REV$"):match("%d+")) or 0

-- Dependencies
if not LibStub then error(MAJOR .. " requires LibStub.") end
if not LibStub("LibSimpleTimer-1.0") then error(MAJOR .. " requires LibSimpleTimer-1.0.") end
if not LibStub("CallbackHandler-1.0") then error(MAJOR .. " requires CallbackHandler-1.0.") end


-------------------------------------------------------------------------------
-- Library - Register & Mixin definition
-------------------------------------------------------------------------------
-- Register
local L 				= _G.LibAvionEmbed__L
local oldLib 			= LibStub:GetLibrary(MAJOR, true)
local oldLibEmbed 		= LibStub:GetLibrary(MAJOR_EMBED, true)
local LibAvion 			= LibStub:NewLibrary(MAJOR, MINOR)
local LibAvionEmbed		= LibStub:NewLibrary(MAJOR_EMBED, MINOR)
if not LibAvion then return end

-- Set as mixin
LibAvionEmbed.mixinTargets = oldLibEmbed and oldLibEmbed.mixinTargets or LibAvionEmbed.mixinTargets or {}
local mixins = {
	"NewAnimation",
	"DelAnimation",
	"AnimationIterator",
	"GetAvionOptionTable",
	"LinkAnimationDB",
	"TriggerAnimationByID",
	
	"BuildAvionCoreOption",
	"BuildAnimationOption",
	"GetTestFrameStatus",
	"SetTestFrameStatus",
	"IsTestFrameShown",
	"HideTestFrame",
	"ShowTestFrame",
	"SliderListener",
	"MoveListener",
	"ResizeListener",
	"MouseDownListener",
	"MouseUpListener",
	
	"callbacks"
}

-- Used for embedding
function LibAvionEmbed:Embed(target)
	for _, name in pairs(mixins) do
		target[name] = LibAvionEmbed[name]
	end
	
	LibAvionEmbed.mixinTargets[target] = true
end

	
-------------------------------------------------------------------------------
-- Library - Load dependencies
-------------------------------------------------------------------------------
local _Timer 			= LibStub:GetLibrary("LibSimpleTimer-1.0")
local _CallbackHandler 	= LibStub:GetLibrary("CallbackHandler-1.0")
LibAvionEmbed.callbacks	= LibAvionEmbed.callbacks or _CallbackHandler:New(LibAvionEmbed)


-------------------------------------------------------------------------------
-- Library - Variables (Descr. at end of file)
-------------------------------------------------------------------------------
-- "Global" (to all addons)
LibAvion.vars = {}
LibAvion.vars.animationCache 			= oldLib and oldLib.vars.animationCache or 			{}
LibAvion.vars.updateInterval 			= oldLib and oldLib.vars.updateInterval or 			0.05
LibAvion.vars.frameCache 				= oldLib and oldLib.vars.frameCache or 				{}
LibAvion.vars.frameCounter 				= oldLib and oldLib.vars.frameCounter or 			1
LibAvion.vars.guideFrameCache 			= oldLib and oldLib.vars.guideFrameCache or 		{}
LibAvion.vars.guideFrameCacheCounter	= oldLib and oldLib.vars.guideFrameCacheCounter or 	1
LibAvion.vars.animationTableCache		= oldLib and oldLib.vars.animationTableCache or 	{}
LibAvion.vars.funcCache					= oldLib and oldLib.vars.funcCache or 				{}
LibAvion.vars.media 					= oldLib and oldLib.vars.media or 					{}
LibAvion.vars.media.func 				= oldLib and oldLib.vars.func or 					{
	["linear"] = "param",
	["square"] = "param*param",
	["squareroot"] = "math.sqrt(param)",
}
LibAvion.vars.media.texture 			= oldLib and oldLib.vars.texture or 				{
	["Minimap"] = [[Interface\WorldMap\UI-World-Icon]],
}
LibAvion.vars.media.sound 				= oldLib and oldLib.vars.sound or 					{
	["None"] = false,
	["Levelup"] = [[Sound\interface\levelup2.wav]],
}


-------------------------------------------------------------------------------
-- Library - Core Animation
-------------------------------------------------------------------------------
-- Create a new frame
local function MakeFrame()
	local self = LibAvion
	
	-- Name of the frame
	local frameName = "Avion_Frame_" .. self.vars.frameCounter

	-- Create a new frame and apply some basic settings
	local frame = CreateFrame("FRAME", frameName, UIParent)
	frame:Hide()
	frame:SetPoint("CENTER", UIParent, "CENTER")
	frame:SetBackdropColor(0, 0, 0, 0)
	frame:SetFrameStrata("BACKGROUND")
	frame:SetScale(1)
	
	-- Add a texture to the frame (Blend mode = additive)
	local texture = frame:CreateTexture(frameName .. "_Texture", "BACKGROUND")
	texture:SetParent(frame)
	texture:SetAllPoints(frame)
	texture:SetBlendMode("ADD")
	
	-- Add everything to the cache
	self.vars.frameCache[self.vars.frameCounter] = {}
	self.vars.frameCache[self.vars.frameCounter].inUse = true
	self.vars.frameCache[self.vars.frameCounter].frame = frame
	self.vars.frameCache[self.vars.frameCounter].texture = texture
	self.vars.frameCache[self.vars.frameCounter].elapsedTimes = 0
	
	-- Increment number of frames and return new number
	self.vars.frameCounter = self.vars.frameCounter + 1
	return (self.vars.frameCounter-1)
end


-- Get an unused frame or create a new one (only ID)
local function GetAvailableFrameID()
	-- Is there any unused/invisible frame?
	for index, cache in pairs(LibAvion.vars.frameCache) do
		if (not cache.inUse) then
			-- Enable inUse and return index
			cache.inUse = true
			cache.elapsedTimes = 0
			
			return index
		end
	end
	
	-- If not, create one!
	return MakeFrame()
end


-- Get an unused frame or create a new one
local function GetAvailableFrame()
	local frameID = GetAvailableFrameID()
	return LibAvion.vars.frameCache[frameID], frameID
end


-- Update values with linear interporlation between start and end
local function Interpolation(startValues, endValues, param) 
	return startValues + (endValues - startValues) * param
end


-- Will rotate texture (and shrink it by a factor of sqrt(2)) and mirror at X/Y axis if needed
local function RotateAndMirror(texture, angle, mirrorX, mirrorY)
	--[[
	NOTE 1: 
	You want to rotate the rectangle (0,0), (1,0), (1,1), (0,1) arround the Point (0.5, 0.5).
	The normal 2D-Rotationsmatrix
	R = [cos(a) -sin(a); sin(a) cos(a)] (where ; means new row)
	will rotate erything around the oripgin point (0,0).
	Thus we need to do the following:
	Traslate our desired rotation-point (0.5,0.5) into the origin by
	T = [1 0 -0.5; 0 1 -0.5; 0 0 1] (in homogenous coords, rougly meaning [x y] will get [x y 1])
	Now we can use the above R (also in homogenous coords) to rotate around the origin and then use
	T^-1 = [1 0 0.5; 0 1 0.5; 0 0 1] 
	to move everything back to were it belongs.
	Given our texture coords [x y]_texture this would lead to:
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * [cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture
	which (in short) can be seen below.
	
	NOTE 2:
	There is just one tiny problem, after each rotation the coordibates still have to be
	inside the rectangle (0,0), (1,0), (1,1), (0,1)  or some parts of the texture won't be
	displayed. Thus, the all texture coords (texture NOT frame) are "scaled down"
	by a factor of sqrt(2) (which is, after pytagoras the length of the recangles diagonal).
	Yet scaling is also done around the origin, not (0.5, 0.5), thus we used the exact same
	procedure as above, only replacinf R with
	S = [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]
	leaving:
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture
	
	NOTE 3:
	Mirroring is done by
	M_x = [-1 0 0; 0 1 0; 0 0 1]
	M_y = [1 0 0; 0 -1 0; 0 0 1]
	Some rules as above apply to mirroring. Need to mirror around X=0.5 or Y=0.5 axis, but those matrizes only
	mirror around X- or Y-axis, leading to:
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * M_x * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture (X Axis)
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * M_y * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture (Y Axis)
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * M_x * M_y * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture (X=Y Axis)
	
	NOTE 4:
	For final result, all transformations need to be combined (in the right order, since matrix multiplication
	is commonly not kummulativ):
	[x y1]_transformed 
	= [1 0 0.5; 0 1 0.5; 0 0 1]  * [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1]  (scale)
	* [1 0 0.5; 0 1 0.5; 0 0 1]  * [cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1]  (rotate)
	* [1 0 0.5; 0 1 0.5; 0 0 1]  * M_xORy * [1 0 -0.5; 0 1 -0.5; 0 0 1]  (mirror)
	* [x y 1]_texture
	Which is equal to:
	x y1]_transformed 
	= [1 0 0.5; 0 1 0.5; 0 0 1]  (move back from origin)
	* [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]  (scale)
	* [cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1]  (rotate)
	* M_xORy (mirror)
	* [1 0 -0.5; 0 1 -0.5; 0 0 1]  (move to origin)
	* [x y 1]_texture
	Apply this transformations to all edges of the (0,0), (1,0), (1,1), (0,1) rectangle, you SHOULD
	get the values you see below. (0.5 * sqrt(2) = 1/sqrt(2) ~ 0.7072)
	
	NOTE 5:
	Remember that texture origin is at top-left of frame. Meaning x goes right, y goes down.
	]]
	
	-- Mirror
	local mX, mY = 1, 1
	if (mirrorX) then
		mX = -1
	end
	if (mirrorY) then
		mY = -1
	end
	
	-- Rotation
	local cosAlpha = 0.7072 * math.cos(angle)
	local sinAlpha = 0.7072 * math.sin(angle)
	texture:SetTexCoord(
		-cosAlpha*mY + sinAlpha*mX + 0.5, -sinAlpha*mY - cosAlpha*mX + 0.5, 
		-cosAlpha*mY - sinAlpha*mX + 0.5, -sinAlpha*mY + cosAlpha*mX + 0.5, 
		 cosAlpha*mY + sinAlpha*mX + 0.5,  sinAlpha*mY - cosAlpha*mX + 0.5, 
		 cosAlpha*mY - sinAlpha*mX + 0.5,  sinAlpha*mY + cosAlpha*mX + 0.5
	)
end


-- Initialize frame settings and show it
local function InitFrame(object, animationTable)
	-- (Re)set texture file and color
	object.texture:SetTexture(animationTable.textureFile)
	object.texture:SetVertexColor(animationTable.color.r, animationTable.color.g, animationTable.color.b)
	
	-- Adjust frame size, position and alpha
	object.frame:SetHeight(animationTable.size.height * 1.4142)
	object.frame:SetWidth(animationTable.size.width * 1.4142)
	object.frame:SetPoint("CENTER", UIParent, "CENTER", animationTable.position.x, animationTable.position.y)
	object.frame:SetAlpha(animationTable.alpha)
	
	-- Rotation
	RotateAndMirror(object.texture, animationTable.rotation, animationTable.mirror.x, animationTable.mirror.y)
	
	-- Finally show it
	object.frame:Show()
end


-- Actually update frame animation
local function AnimateFrame(object, animationTable, param)
	-- Get and set updated values
	if (animationTable.isAnimated.position) then
		local position = {
			x = Interpolation(
				animationTable.position.x, 
				animationTable.animation.position.x, 
				animationTable.animationFunc.position(param, animationTable.addRandom.position)
			),
			y = Interpolation(
				animationTable.position.y, 
				animationTable.animation.position.y, 
				animationTable.animationFunc.position(param, animationTable.addRandom.position)
			)
		}
		object.frame:SetPoint("CENTER", UIParent, "CENTER", position.x, position.y)
	end
	if (animationTable.isAnimated.scale) then
		local scale = Interpolation(
			1, 
			animationTable.animation.scale, 
			animationTable.animationFunc.scale(param, animationTable.addRandom.scale)
		)
		object.frame:SetHeight(animationTable.size.height * 1.4142 * scale)
		object.frame:SetWidth(animationTable.size.width * 1.4142 * scale)
	end
	if (animationTable.isAnimated.color) then
		local color = {
			r = Interpolation(
				animationTable.color.r, 
				animationTable.animation.color.r, 
				animationTable.animationFunc.color(param, animationTable.addRandom.color)
			),
			g = Interpolation(
				animationTable.color.g, 
				animationTable.animation.color.g, 
				animationTable.animationFunc.color(param, animationTable.addRandom.color)
			),
			b = Interpolation(
				animationTable.color.b, 
				animationTable.animation.color.b, 
				animationTable.animationFunc.color(param, animationTable.addRandom.color)
			)
		}
		object.texture:SetVertexColor(color.r, color.g, color.b)
	end
	if (animationTable.isAnimated.rotation) then
		local angle = Interpolation(
			animationTable.rotation, 
			animationTable.animation.rotation,
			animationTable.animationFunc.rotation(param, animationTable.addRandom.rotation)
		)
		RotateAndMirror(object.texture, angle, animationTable.mirror.x, animationTable.mirror.y)
	end	
end


-- Update frame animation
local function UpdateAnimation(object, animationTable, forceUpdate, fadeOnly)
	-- Update elapsed time (time since animation was triggered)
	local elapsedTime = object.elapsedTimes * LibAvion.vars.updateInterval
	object.elapsedTimes = object.elapsedTimes + 1
	
	-- Parameter for update function
	local param
	
	-- Animate frame, move, resize, color, rotate, ect.
	if (elapsedTime <= animationTable.animationTime) then
		-- Calc parameter for linear interpolation (param \in [0...1]) and animate frame
		if (animationTable.animateAndFade) then
			param = elapsedTime / (animationTable.animationTime + animationTable.fadeTime)
		else
			param = elapsedTime / animationTable.animationTime
		end
		AnimateFrame(object, animationTable, param)
		
	-- Stop doing anything to the frame
	elseif (not fadeOnly and animationTable.noEnd) then
		-- Stop animation timer, since nothing to do
		if (_Timer:IsTimerScheduled("Avion_UpdateAnimation_" .. object.triggerID)) then
			_Timer:ScheduleTimer("Avion_UpdateAnimation_" .. object.triggerID, nil, 0)
		end
	
	-- Start fading frame
	elseif(elapsedTime - animationTable.animationTime <= animationTable.fadeTime) then	
		-- Calc parameter for linear interpolation (param \in [0...1]) and continue animation
		if (not fadeOnly and animationTable.animateAndFade) then
			param = elapsedTime / (animationTable.animationTime + animationTable.fadeTime)
			
			AnimateFrame(object, animationTable, param)
		elseif (forceUpdate) then
			AnimateFrame(object, animationTable, 1)
		end

		-- Fade frame out
		param = (elapsedTime - animationTable.animationTime)/ animationTable.fadeTime
		object.frame:SetAlpha(
			Interpolation(
				animationTable.alpha, 
				0, 
				animationTable.animationFunc.alpha(param, animationTable.addRandom.alpha)
			)
		)
	
	-- Done, Unregister
	else
		LibAvion:StopAnimation(object.triggerID, false)
	end
end


-- "Register" a new frame for animation (this registration handels timing)
local function StartAnimationTimer(object, animationTable, animationTime, waitTime)
	local self = LibAvion
	
	-- If there is NO wait-time, initialize and run animation
	if (waitTime == 0) then
		-- Initialize frame stuff
		if (not object.frame:IsVisible()) then
			InitFrame(object, animationTable)
		end
		
		-- We only need to test if something is animated if there is really animation-time ^^
		local anythingAnimated = false
		if (animationTime ~= 0) then
			-- If there is nothing to animate, well ... life is easier ^^
			for typ, value in pairs(animationTable.isAnimated) do
				if (value) then
					anythingAnimated = true
					break
				end
			end
		end
		
		-- Well ... thats life, go the hard way ^^
		if (animationTime == 0 or anythingAnimated) then
			-- Register animation
			_Timer:ScheduleRepeatingTimer(
				"Avion_UpdateAnimation_" ..object.triggerID,
				UpdateAnimation,
				self.vars.updateInterval,
				--self, 
				object, 
				animationTable,
				false,
				false
			)
		else
			-- Start animation after animationTime has passed, since there was nothing to animate...
			_Timer:ScheduleTimer(
				"Avion_DisplayAnimation_" .. object.triggerID,
				StartAnimationTimer,
				animationTable.animationTime, 
				--self, 
				object, 
				animationTable, 
				0, 
				0
			)
		end
	-- If there IS wait-time, retrigger after waitTime-delay
	else
		-- Show frame allready?
		if (animationTable.showWhileWait) then
			InitFrame(object, animationTable)
		end
		
		-- Start waiting
		_Timer:ScheduleTimer(
			"Avion_WaitAnimation_" .. object.triggerID,
			StartAnimationTimer,
			animationTable.waitTime,  
			--self, 
			object, 
			animationTable, 
			animationTime, 
			0
		)
	end
end


-------------------------------------------------------------------------------
-- Library - Misc
-------------------------------------------------------------------------------
-- Create a deep copy of an animationTable
local function DeepAnimationTableCopy(animationTable)
	return {
		name 			= animationTable.name,
		alpha 			= animationTable.alpha,
		color 			= {
			r				= animationTable.color.r, 
			g 				= animationTable.color.g, 
			b 				= animationTable.color.b
		},
		position 		= {
			x 				= animationTable.position.x, 
			y 				= animationTable.position.y
		},
		size 			= {
			width 			= animationTable.size.width, 
			height 			= animationTable.size.height
		},
		rotation 		= animationTable.rotation,
		isAnimated 		= {
			color 			= animationTable.isAnimated.color, 
			position 		= animationTable.isAnimated.position, 
			scale 			= animationTable.isAnimated.scale, 
			rotation 		= animationTable.isAnimated.rotation
		},
		animation 		= {
			color 			= {
				r 				= animationTable.animation.color.r, 
				g 				= animationTable.animation.color.g, 
				b 				= animationTable.animation.color.b
			}, 
			position 		= {
				x 				= animationTable.animation.position.x, 
				y 				= animationTable.animation.position.y
			}, 
			scale 			= animationTable.animation.scale, 
			rotation 		= animationTable.animation.rotation
		},
		animationFunc	= {
			alpha 			= animationTable.animationFunc.alpha, 
			color 			= animationTable.animationFunc.color, 
			position 		= animationTable.animationFunc.position, 
			scale 			= animationTable.animationFunc.scale, 
			rotation 		= animationTable.animationFunc.rotation
		},
		addRandom = {
			alpha 			= animationTable.addRandom.alpha, 
			color 			= animationTable.addRandom.color, 
			position 		= animationTable.addRandom.position, 
			scale 			= animationTable.addRandom.scale, 
			rotation 		= animationTable.addRandom.rotation
		},
		animateAndFade 	= animationTable.animateAndFade,
		animationTime 	= animationTable.animationTime,
		fadeTime 		= animationTable.fadeTime,
		waitTime 		= animationTable.waitTime,
		showWhileWait 	= animationTable.showWhileWait,
		textureFile 	= animationTable.textureFile,
		mirror			= {
			x 				= animationTable.mirror.x,
			y				= animationTable.mirror.y,
		},
		soundFile 		= animationTable.soundFile,
		noEnd 			= animationTable.noEnd
	}
end


-- Convert from Database format (easyly saveable) to inter-use format
local function AnimationInternToExtern(animationListEntry)
	-- Make a deep copy first
	local animationTable = DeepAnimationTableCopy(animationListEntry)
	
	-- But we have to prepare the table a bit
	if (animationTable.soundFile) then
		animationTable.soundFile = LibAvion.vars.media.sound[animationTable.soundFile]
	else
		animationTable.soundFile = nil
	end
	if (animationTable.textureFile) then
		animationTable.textureFile = LibAvion.vars.media.texture[animationTable.textureFile]
	else
		animationTable.textureFile = [[Interface\Icons\Temp.blp]]
	end
	animationTable.animationFunc = {
		alpha 		= LibAvion.vars.funcCache[animationTable.animationFunc.alpha], 
		color 		= LibAvion.vars.funcCache[animationTable.animationFunc.color], 
		position 	= LibAvion.vars.funcCache[animationTable.animationFunc.position], 
		scale 		= LibAvion.vars.funcCache[animationTable.animationFunc.scale], 
		rotation 	= LibAvion.vars.funcCache[animationTable.animationFunc.rotation]
	}

	return animationTable
end


-- Create (real) function from SharedMedia/String entry
local function MediaToFunc(key)
	local func = assert(loadstring([[
		return function (param, rnd) 
			if (rnd) then
				return (]] .. (LibAvion.vars.media.func[key] or "0") ..[[ + math.random(-rnd, rnd) / 10000) 
			else 
				return (]] .. (LibAvion.vars.media.func[key] or "0") .. [[)
			end
		end
	]]))
	local result = func()
	
	return result
end


-------------------------------------------------------------------------------
-- Library - API - Lightweight
-------------------------------------------------------------------------------
-- "Register" a new frame for animation (this registration handels settings)
function LibAvion:TriggerAnimation(triggerID, animationTable)
	-- Quite when disabled or allready registered
	if (self:IsAnimationRunning(triggerID)) then
		return
	end
	
	-- Backup animation data
	self.vars.animationTableCache[triggerID] = animationTable
	
	-- Make things local
	local object
	object, self.vars.animationCache[triggerID] = GetAvailableFrame()
	
	-- Save which triggerID triggerd this animation
	object.triggerID = triggerID
	
	-- Play soundfile if there is one to play
	if (animationTable.soundFile) then
		PlaySoundFile(animationTable.soundFile)
	end
	
	-- Finally "start" animation/timer
	StartAnimationTimer(object, animationTable, animationTable.animationTime, animationTable.waitTime)
end


-- "Unregister" animation
function LibAvion:StopAnimation(triggerID, fadeOut)
	-- Hmmm, well trying to unregister an unregister animation may be painful >:>
	if (self:IsAnimationRunning(triggerID)) then
		-- Unregister animation (delayed and repeating animations)
		if (_Timer:IsTimerScheduled("Avion_UpdateAnimation_" .. triggerID)) then
			_Timer:ScheduleTimer("Avion_UpdateAnimation_" .. triggerID, nil, 0)
		end
		if (_Timer:IsTimerScheduled("Avion_DisplayAnimation_" .. triggerID)) then
			_Timer:ScheduleTimer("Avion_DisplayAnimation_" .. triggerID, nil, 0)
		end
		if (_Timer:IsTimerScheduled("Avion_WaitAnimation_" .. triggerID)) then
			_Timer:ScheduleTimer("Avion_WaitAnimation_" .. triggerID, nil, 0)
		end
		
		-- Smoothly fade-out frame
		if (fadeOut) then
			-- Needed vars
			local object 			= LibAvion.vars.frameCache[self.vars.animationCache[triggerID]]
			local animationTable	= self.vars.animationTableCache[triggerID]
			
			-- Adjust run-time of this animation to fit fading
			if (object.elapsedTimes * LibAvion.vars.updateInterval - animationTable.animationTime < animationTable.FadeTime) then
				object.elapsedTimes = animationTable.animationTime / LibAvion.vars.updateInterval
			end
			
			-- Register for fading animation
			_Timer:ScheduleRepeatingTimer(
				"Avion_UpdateAnimation_" .. triggerID,
				UpdateAnimation, 
				LibAvion.vars.updateInterval,
				--self, 
				object, 
				animationTable,
				false,
				true
			)
		
		-- Stop and hide animation
		else
			-- Hide frame and its not inUse anymore...
			self.vars.frameCache[self.vars.animationCache[triggerID]].texture:SetTexture("Interface\\Icons\\Temp.blp")
			self.vars.frameCache[self.vars.animationCache[triggerID]].frame:Hide()
			self.vars.frameCache[self.vars.animationCache[triggerID]].inUse = false
			
			-- Delete frame from animation cache
			self.vars.animationCache[triggerID] 		= nil
			self.vars.animationTableCache[triggerID]	= nil
		end
	end
end


-- "Unregister" ALL animations
function LibAvion:StopAllAnimations(fadeOut)
	if (self.vars.animationCache) then
		for triggerID, _ in pairs(self.vars.animationCache) do
			self:StopAnimation(triggerID, fadeOut)
		end
	end
end


-- Take a look, if Animation isn't allready registered
function LibAvion:IsAnimationRunning(triggerID)
	if (self.vars.animationCache and self.vars.animationCache[triggerID]) then
		return true
	else
		return false
	end
end


-------------------------------------------------------------------------------
-- Library - API - Media Database
-------------------------------------------------------------------------------
-- Add new Texture to DB
function LibAvion:AddTexture(key, value)
	-- Terminate if allready something with that name
	if (self.vars.media.texture[key]) then
		return
	end
	
	-- Terminate if already entry with that value
	for _, _value in pairs(self.vars.media.texture) do
		if (_value == value) then
			return
		end
	end
	
	-- Add me
	self.vars.media.texture[key] = value
end


-- Add new Sound to DB
function LibAvion:AddSound(key, value)
	-- Terminate if allready something with that name
	if (self.vars.media.sound[key]) then
		return
	end
	
	-- Terminate if already entry with that value
	for _, _value in pairs(self.vars.media.sound) do
		if (_value == value) then
			return
		end
	end
	
	-- Add me
	self.vars.media.sound[key] = value
end


-- Add new Function to DB
function LibAvion:AddFunction(key, value)
	-- Terminate if allready something with that name
	if (self.vars.media.func[key]) then
		return
	end
	
	-- Terminate if already entry with that value
	for _, _value in pairs(self.vars.media.func) do
		if (_value == value) then
			return
		end
	end
	
	-- Add me
	self.vars.media.func[key] = value
	LibAvion.vars.funcCache[name] = MediaToFunc(name)
end


-------------------------------------------------------------------------------
-- Library - API - Animation Database
-------------------------------------------------------------------------------
-- Link addonDB to lib
function LibAvionEmbed:LinkAnimationDB(animationList)
	-- "Local" (for each addon)
	if (not self.vars) then
		self.vars 					= {}
	end
	if (not self.vars.linkCache) then
		self.vars.linkCache			= {}
	end
	
	-- Load (default) animation functions into mem
	for name, _ in pairs(LibAvion.vars.media.func) do
		LibAvion.vars.funcCache[name] = MediaToFunc(name)
	end

	self.vars.animationList = animationList
end


-- Iterator that returns id and name for each animation
function LibAvionEmbed:AnimationIterator()
	local id
	
	return function ()
		id = next(self.vars.animationList, id)

		-- we reached the end early?
		if id == nil then
			return nil
		end
		
		while self.vars.animationList[id] == nil do
			id = next(self.vars.animationList, id)
			
			if id == nil then
				return nil
			end
		end
		
		return id, self.vars.animationList[id].name
	end
end


-- Build Optionstable
function LibAvionEmbed:GetAvionOptionTable()
	-- Initialize it
	if not self.vars.OnMenuRequest then
		self.vars.OnMenuRequest = self:BuildAvionCoreOption()
	end
	
	-- Build option-table for each animation
	self.vars.OnMenuRequest.edit.args = nil
	self.vars.OnMenuRequest.edit.args = { }
	for animationID, _ in pairs(self.vars.animationList) do
		self.vars.OnMenuRequest.edit.args[animationID] = self:BuildAnimationOption(animationID)
	end
	
	-- GuideFrames updaten
	for animationID, _ in pairs(self.vars.linkCache) do
		self:MouseDownListener(true, animationID)
	end
	
	-- Trigger Event
	self.callbacks:Fire("LibAvion_MenuUpdate")
	
	return self.vars.OnMenuRequest
end


-- Add new animation (from default-db or existing one)
function LibAvionEmbed:NewAnimation(animationTable, newName)
	local animationID = #(self.vars.animationList) + 1
	
	-- Name should be a String and longer than zero chars
	if (strlen(newName) == 0) then
		return false
	end
	
	-- Default or copy
	if (animationTable ~= nil) then
		self.vars.animationList[animationID] = DeepAnimationTableCopy(animationTable)
		self.vars.animationList.name = newName
	else
		self.vars.animationList[animationID] = {
			name = newName,
			alpha = 1,
			color = {r = 1, g = 1, b = 1},
			position = {x = 0, y = 0},
			size = {width = 200, height = 200},
			rotation = 0,
			isAnimated = {color = true, position = false, scale = true, rotation = false },
			animation = {color = {r = 1, g = 0, b = 0}, position = {x = 0, y = 0}, scale = 2, rotation = 0},
			animationFunc = {alpha = "linear", color = "linear", position = "linear", scale = "linear", rotation = "linear"},
			addRandom = {alpha = 100, color = 100, position = 100, scale = 100, rotation = 100},
			animateAndFade = false,
			animationTime = 3,
			fadeTime = 2,
			waitTime = 0,
			showWhileWait = true,
			textureFile = "Minimap",
			soundFile = "None",
			mirror = {
				x = false,
				y = false,
			},
			noEnd = false
		}
	end
	
	-- Refresh 
	self:GetAvionOptionTable()
	
	-- Trigger Event
	self.callbacks:Fire("LibAvion_NewAnimation", animationID)
	
	return animationID
end


-- Delete animation from db
function LibAvionEmbed:DelAnimation(animationID)
	-- Stop running animations and delete
	LibAvion:StopAllAnimations()
	self.vars.animationList[animationID] = nil

	-- Refresh
	self:GetAvionOptionTable()
	
	-- Trigger Event
	self.callbacks:Fire("LibAvion_DeleteAnimation", animationID)
	
	return animationID
end


-- Trigger Animation with certain animationID
function LibAvionEmbed:TriggerAnimationByID(triggerID, animationID)
	-- Make a deep copy
	local animationTable = AnimationInternToExtern(self.vars.animationList[animationID])

	-- Trigger it
	LibAvion:TriggerAnimation(triggerID, animationTable)
end


-------------------------------------------------------------------------------
-- Library - Options
-------------------------------------------------------------------------------
-- Default options
function LibAvionEmbed:BuildAvionCoreOption() 
	return {
		edit = {
			order = 1,
			type = 'group',
			name = L["EditAnimation"],
			desc = L["EditAnimation_Desc"],
		},
		new = {
			order = 2,
			type = 'string',
			name = L["NewAnimation"],
			desc = L["NewAnimation_Desc"],
			usage = L["NewAnimation_Usage"],
			get = function()
				return ""
			end,
			set = function(new)
				self:NewAnimation(nil, new)
			end,
			validate = function(new)
				return (strlen(new) > 0)
			end,
		},
	}
end


-- Build option-table for given animation
function LibAvionEmbed:BuildAnimationOption(animationID)
	if (self.vars.animationList[animationID]) then
		-- Build a list of animations to copy data from
		local copyTable = {}
		for _animationID, _ in pairs(self.vars.animationList) do
			if (_animationID ~= animationID) then
				copyTable[_animationID] = _animationID
			end
		end
		
		local funcList = {}
		for _name, _ in pairs(LibAvion.vars.media.func) do
			funcList[_name] = _name
		end
		
		local textureList = {}
		for _name, _ in pairs(LibAvion.vars.media.texture) do
			textureList[_name] = _name
		end
		
		local soundList = {}
		for _name, _ in pairs(LibAvion.vars.media.sound) do
			soundList[_name] = _name
		end
		
		-- Return OptionsTable
		return {
			order = animationID,
			type = 'group',
			name = self.vars.animationList[animationID].name,
			desc = L["AnimationOption_Desc"],
			args = {
				name = {
					order = 1,
					type = 'string',
					name = L["Name"],
					desc = L["Name_Desc"],
					usage = L["Name_Usage"],
					get = function()
						return self.vars.animationList[animationID].name
					end,
					set = function(newName)
						self.vars.animationList[animationID].name = newName
						
						self:GetAvionOptionTable()
					end,
				},
				startValues = {
					order = 2,
					type = 'group',
					name = L["StartValues"],
					desc = L["StartValues_Desc"],
					args = { 
						color = {
							order = 1,
							type = 'color',
							name = L["ColorAlpha"],
							desc = L["ColorAlpha_Desc"],
							get = function()
								return self.vars.animationList[animationID].color.r, self.vars.animationList[animationID].color.g, self.vars.animationList[animationID].color.b, self.vars.animationList[animationID].alpha
							end,
							set = function(r, g, b, a)
								self.vars.animationList[animationID].color.r = r
								self.vars.animationList[animationID].color.g = g
								self.vars.animationList[animationID].color.b = b
								self.vars.animationList[animationID].alpha = a
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							hasAlpha = true
						},
						positionX = {
							order = 2,
							type = 'number',
							name = L["PositionX"],
							desc = L["PositionX_Desc"],
							get = function()
								return self.vars.animationList[animationID].position.x
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].position.x = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						positionY = {
							order = 3,
							type = 'number',
							name = L["PositionY"],
							desc = L["PositionY_Desc"],
							get = function()
								return self.vars.animationList[animationID].position.y
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].position.y = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						width = {
							order = 4,
							type = 'number',
							name = L["Width"],
							desc = L["Width_Desc"],
							get = function()
								return self.vars.animationList[animationID].size.width
							end,
							set = function(newWidth)
								self.vars.animationList[animationID].size.width = newWidth
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale()),
							step = 1,
						},
						height = {
							order = 5,
							type = 'number',
							name = L["Height"],
							desc = L["Height_Desc"],
							get = function()
								return self.vars.animationList[animationID].size.height
							end,
							set = function(newHeight)
								self.vars.animationList[animationID].size.height = newHeight
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max =  math.floor(GetScreenHeight() * 1 / UIParent:GetScale()),
							step = 1,
						},
						rotation = {
							order = 6,
							type = 'number',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								return self.vars.animationList[animationID].rotation / math.pi * 180
							end,
							set = function(newRotation)
								self.vars.animationList[animationID].rotation = newRotation * math.pi / 180
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 359,
							step = 1,
						},
					},
				},
				animation = {
					order = 3,
					type = 'group',
					name = L["AnimationValues"],
					desc = L["AnimationValues_Desc"],
					args = { 
						color = {
							order = 1,
							type = 'color',
							name = L["ColorAlpha"],
							desc = L["ColorAlpha_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.color.r, self.vars.animationList[animationID].animation.color.g, self.vars.animationList[animationID].animation.color.b
							end,
							set = function(r, g, b, a)
								self.vars.animationList[animationID].animation.color.r = r
								self.vars.animationList[animationID].animation.color.g = g
								self.vars.animationList[animationID].animation.color.b = b
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
						positionX = {
							order = 2,
							type = 'number',
							name = L["PositionX"],
							desc = L["PositionX_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.position.x
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].animation.position.x = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						positionY = {
							order = 3,
							type = 'number',
							name = L["PositionY"],
							desc = L["PositionY_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.position.y
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].animation.position.y = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						rotation = {
							order = 4,
							type = 'number',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.rotation / math.pi * 180
							end,
							set = function(newRotation)
								self.vars.animationList[animationID].animation.rotation = newRotation * math.pi / 180
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 359,
							step = 1,
						},
						scale = {
							order = 5,
							type = 'number',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.scale
							end,
							set = function(newScale)
								self.vars.animationList[animationID].animation.scale = newScale
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0.1,
							max = 10,
							step = 0.05,
						},
					},
				},
				animationFunc = {
					order = 4,
					type = 'group',
					name = L["AnimationFunc"],
					desc = L["AnimationFunc_Desc"],
					args = { 
						alpha = {
							order = 1,
							type = 'choice',
							name = L["Alpha"],
							desc = L["Alpha_Desc"],
							choices = funcList,
							choiceType = 'dict',
							get = function()
								return self.vars.animationList[animationID].animationFunc.alpha
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.alpha = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
						color = {
							order = 2,
							type = 'choice',
							name = L["Color"],
							desc = L["Color_Desc"],
							choices = funcList,
							choiceType = 'dict',
							get = function()
								return self.vars.animationList[animationID].animationFunc.color
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.color = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
						position = {
							order = 3,
							type = 'choice',
							name = L["Position"],
							desc = L["Position_Desc"],
							choices = funcList,
							choiceType = 'dict',
							get = function()
								return self.vars.animationList[animationID].animationFunc.position
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.position = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
						scale = {
							order = 4,
							type = 'choice',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							choices = funcList,
							choiceType = 'dict',
							get = function()
								return self.vars.animationList[animationID].animationFunc.scale
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.scale = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
						rotation = {
							order = 5,
							type = 'choice',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							choices = funcList,
							choiceType = 'dict',
							get = function()
								return self.vars.animationList[animationID].animationFunc.rotation
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.rotation = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
					},
				},
				addRandom = {
					order = 5,
					type = 'group',
					name = L["AddRandom"],
					desc = L["AddRandom_Desc"],
					args = {
						color = {
							order = 1,
							type = 'number',
							name = L["Color"],
							desc = L["Color_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.color) then
									return self.vars.animationList[animationID].addRandom.color
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.color = nil
								else
									self.vars.animationList[animationID].addRandom.color = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
						position = {
							order = 2,
							type = 'number',
							name = L["Position"],
							desc = L["Position_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.position) then
									return self.vars.animationList[animationID].addRandom.position
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.position = nil
								else
									self.vars.animationList[animationID].addRandom.position = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
						scale = {
							order = 3,
							type = 'number',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.scale) then
									return self.vars.animationList[animationID].addRandom.scale
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.scale = nil
								else
									self.vars.animationList[animationID].addRandom.scale = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
						rotation = {
							order = 4,
							type = 'number',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.rotation) then
									return self.vars.animationList[animationID].addRandom.rotation
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.rotation = nil
								else
									self.vars.animationList[animationID].addRandom.rotation = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
					},
				},
				isAnimated = {
					order = 6,
					type = 'group',
					name = L["IsAnimated"],
					desc = L["IsAnimated_Desc"],
					args = {
						color = {
							order = 1,
							type = 'boolean',
							name = L["Color"],
							desc = L["Color_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.color
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.color = toggle
								
								-- GuideFrames updaten
								for animationID, _ in pairs(self.vars.linkCache) do
									self:MouseDownListener(true, animationID)
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
						position = {
							order = 2,
							type = 'boolean',
							name = L["Position"],
							desc = L["Position_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.position
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.position = toggle
								
								-- GuideFrames updaten
								for animationID, _ in pairs(self.vars.linkCache) do
									self:MouseDownListener(true, animationID)
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
						scale = {
							order = 3,
							type = 'boolean',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.scale
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.scale = toggle
								
								-- GuideFrames updaten
								for animationID, _ in pairs(self.vars.linkCache) do
									self:MouseDownListener(true, animationID)
								end

								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
						rotation = {
							order = 4,
							type = 'boolean',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.rotation
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.rotation = toggle
								
								-- GuideFrames updaten
								for animationID, _ in pairs(self.vars.linkCache) do
									self:MouseDownListener(true, animationID)
								end

								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
					},
				},
				timing = {
					order = 7,
					type = 'group',
					name = L["Durations"],
					desc = L["Durations_Desc"],
					args = { 
						waitTime = {
							order = 1,
							type = 'number',
							name = L["WaitTime"],
							desc = L["WaitTime_Desc"],
							get = function()
								return self.vars.animationList[animationID].waitTime
							end,
							set = function(new)
								self.vars.animationList[animationID].waitTime = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
								
								self:GetAvionOptionTable()
							end,
							min = 0,
							max = 30,
							step = LibAvion.vars.updateInterval,
						},
						animationTime = {
							order = 2,
							type = 'number',
							name = L["AnimationTime"],
							desc = L["AnimationTime_Desc"],
							get = function()
								return self.vars.animationList[animationID].animationTime
							end,
							set = function(new)
								self.vars.animationList[animationID].animationTime = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
								
								self:GetAvionOptionTable()
							end,
							min = 0,
							max = 30,
							step = LibAvion.vars.updateInterval,
						},
						fadeTime = {
							order = 3,
							type = 'number',
							name = L["FadeTime"],
							desc = L["FadeTime_Desc"],
							get = function()
								return self.vars.animationList[animationID].fadeTime
							end,
							set = function(new)
								self.vars.animationList[animationID].fadeTime = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
								
								self:GetAvionOptionTable()
							end,
							min = 0,
							max = 30,
							step = LibAvion.vars.updateInterval,
						},
					},
				},
				texture = {
					order = 8,
					type = 'choice',
					name = L["Texture"],
					desc = L["Texture_Desc"],
					choices = textureList,
					choiceType = 'dict',
					get = function()
						return self.vars.animationList[animationID].textureFile
					end,
					set = function(new)
						self.vars.animationList[animationID].textureFile = new
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
				},
				mirrorX = {
					order = 9,
					type = 'boolean',
					name = L["MirrorX"],
					desc = L["MirrorX_Desc"],
					get = function()
						return self.vars.animationList[animationID].mirror.x
					end,
					set = function(toggle)
						self.vars.animationList[animationID].mirror.x = toggle
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
				},
				mirrorY = {
					order = 10,
					type = 'boolean',
					name = L["MirrorY"],
					desc = L["MirrorY_Desc"],
					get = function()
						return self.vars.animationList[animationID].mirror.y
					end,
					set = function(toggle)
						self.vars.animationList[animationID].mirror.y = toggle
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
				},
				sound = {
					order = 11,
					type = 'choice',
					name = L["Sound"],
					desc = L["Sound_Desc"],
					choices = soundList,
					choiceType = 'dict',
					get = function()
						if (self.vars.animationList[animationID].soundFile == nil) then
							return "None"
						else
							return self.vars.animationList[animationID].soundFile
						end
					end,
					set = function(new)
						if (new == "None") then
							self.vars.animationList[animationID].soundFile = nil
						else
							self.vars.animationList[animationID].soundFile = new
							
							PlaySoundFile(LibAvion.vars.media.sound[new])
						end
					end,
				},
				animateAndFade = {
					order = 12,
					type = 'boolean',
					name = L["AnimateAndFade"],
					desc = L["AnimateAndFade_Desc"],
					get = function()
						return self.vars.animationList[animationID].animateAndFade
					end,
					set = function(toggle)
						self.vars.animationList[animationID].animateAndFade = toggle
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
				},
				showWhileWait = {
					order = 13,
					type = 'boolean',
					name = L["ShowWhileWait"],
					desc = L["ShowWhileWait_Desc"],
					get = function()
						return self.vars.animationList[animationID].showWhileWait
					end,
					set = function(toggle)
						self.vars.animationList[animationID].showWhileWait = toggle
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
				},
				noEnd = {
					order = 14,
					type = 'boolean',
					name = L["NoEnd"],
					desc = L["NoEnd_Desc"],
					get = function()
						return self.vars.animationList[animationID].noEnd
					end,
					set = function(toggle)
						self.vars.animationList[animationID].noEnd = toggle
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
				},
				testing = {
					order = 15,
					type = 'group',
					name = L["TestSettings"],
					desc = L["TestSettings_Desc"],
					args = {
						runAnimation = {
							order = 1,
							type = 'execute',
							name = L["RunAnimation"],
							desc = L["RunAnimation_Desc"],
							buttonText = L["RunAnimation"],
							func = function()
								LibAvion:StopAnimation(string.format("%s%d", "Avion_RunAnimation_", animationID), false)
								self:TriggerAnimationByID(string.format("%s%d", "Avion_RunAnimation_", animationID), animationID)
							end
						},
						showHide = {
							order = 2,
							type = 'boolean',
							name = L["Show"],
							desc = L["ShowHide_Desc"],
							get = function()
								return self:IsTestFrameShown(animationID)
							end,
							set = function(toggle)
								if (toggle == true) then
									self:ShowTestFrame(animationID)
								else
									self:HideTestFrame(animationID)
								end
							end,
						},
						elapsedTimes = {
							order = 3,
							type = 'number',
							name = L["Timer"],
							desc = L["Timer_Desc"],
							get = function()
								return self:GetTestFrameStatus(animationID)
							end,
							set = function(new)
								self:SetTestFrameStatus(animationID, new)
							end,
							min = 0,
							max = self.vars.animationList[animationID].waitTime + self.vars.animationList[animationID].animationTime + self.vars.animationList[animationID].fadeTime,
							step = LibAvion.vars.updateInterval,
						},
					},
				},
				delete = {
					order = 16,
					type = 'execute',
					name = L["DeleteAnimation"],
					desc = L["DeleteAnimation_Desc"],
					buttonText = L["DeleteAnimation"],
					func = function()
						self:DelAnimation(animationID)
					end
				},
				copyFrom = {
					order = 17,
					type = 'choice',
					name = L["CopySettings"],
					desc = L["CopySettings_Desc"],
					choices = copyTable,
					get = function()
						return false
					end,
					set = function(key)
						local itsName = self.vars.animationList[animationID].name
						self.vars.animationList[animationID] = nil
						self.vars.animationList[animationID] = DeepAnimationTableCopy(self.vars.animationList[key])
						self.vars.animationList[animationID].name = itsName
						
						self:GetAvionOptionTable()
					end,
				},
			},
		}
	else
		return nil
	end
end


-------------------------------------------------------------------------------
-- Library - Guide Frame
-------------------------------------------------------------------------------
-- Create a new guide frame
local function MakeGuideFrame()
	-- Basic frame
	local frame = CreateFrame("Frame", nil)
	frame:EnableMouse(true)
	frame:SetResizable(true)
	frame:SetMovable(true)
	frame:SetMinResize(22, 22)
	frame:SetFrameStrata("HIGH")
	frame:SetBackdropColor(1, 1, 1, 0)
	
	-- Mouseover highlight and config mode visualisation
	local texture = frame:CreateTexture(nil, "HIGHLIGHT")
	texture:SetAllPoints(frame)
	texture:SetTexture(0, 1, 1)
	texture:SetAlpha(0.3)

	-- Display some descriptive text
	local text = frame:CreateFontString(nil, "HIGHLIGHT")
	text:SetFontObject(GameFontHighlightSmall)
	text:SetPoint("CENTER", frame, "CENTER", 0, 0)
	text:SetText("ERROR")

	-- Resize mouse-drag image
	local resizeTexture = frame:CreateTexture(nil, "HIGHLIGHT")
	resizeTexture:SetHeight(16)
	resizeTexture:SetWidth(16)
	resizeTexture:SetTexture("Interface\\AddOns\\LibAvion-2.0\\resize.tga")
	resizeTexture:SetPoint("BOTTOMRIGHT", -2, 2)
	
	-- Add slider for rotation adjustment
	local slider = CreateFrame("Slider", nil, frame)
	slider:SetOrientation("HORIZONTAL")
	slider:SetWidth(200)
	slider:SetHeight(16)
	slider:SetPoint("BOTTOM", frame, "TOP", 0, 5)
	slider:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
	slider:SetBackdrop({
		bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
		edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
		tile = true,
		edgeSize = 8,
		tileSize = 8,
		insets = {
			left = 3,
			right = 3,
			top = 3,
			bottom= 3
		}
	})
	slider:SetBackdropColor(0, 1, 1, 1)
	slider:SetBackdropBorderColor(0, 1, 1, 1)
	slider:SetMinMaxValues(0, 359)
	slider:SetValue(0)
	slider:SetValueStep(1)
	
	-- Will open options menu for this animation
	local menuButton = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
	menuButton:SetWidth(60)
	menuButton:SetHeight(25)
	menuButton:SetText(L["Menu"])
	menuButton:SetPoint("TOP", frame, "BOTTOM", 0, -5)
	
	-- Button to mirror allong x axis
	local mirrorX = CreateFrame("Button", nil, menuButton, "UIPanelButtonTemplate")
	mirrorX:SetWidth(60)
	mirrorX:SetHeight(25)
	mirrorX:SetText(L["MirrorX"])
	mirrorX:SetPoint("RIGHT", menuButton, "LEFT", 0, 0)
	
	-- Button to mirror allong y axis
	local mirrorY = CreateFrame("Button", nil, menuButton, "UIPanelButtonTemplate")
	mirrorY:SetWidth(60)
	mirrorY:SetHeight(25)
	mirrorY:SetText(L["MirrorY"])
	mirrorY:SetPoint("LEFT", menuButton, "RIGHT", 0, 0)
	
	-- Close Button
	local closeButton = CreateFrame("Button", nil, menuButton, "UIPanelCloseButton")
	closeButton:SetFrameLevel(5)
	closeButton:SetPoint("TOP", menuButton, "BOTTOM", 0, -5)
	
	-- Return result
	return {
		frame 			= frame,
		texture 		= texture,
		text 			= text,
		resizeTexture	= resizeTexture,
		slider 			= slider,
		menuButton 		= menuButton,
		mirrorX 		= mirrorX,
		mirrorY 		= mirrorY,
		closeButton 	= closeButton,
		configMode		= 0
	}
end


-- Will return scale of animation frame, calculated from the guideFrame
local function GetGuideFrameScale(guideObject, animationTable)
	local L1 = sqrt(
		animationTable.size.width  * animationTable.size.width + 
		animationTable.size.height * animationTable.size.height
	)
	local L2 = 1.2 * sqrt(
		guideObject.frame:GetWidth()  * guideObject.frame:GetWidth() + 
		guideObject.frame:GetHeight() * guideObject.frame:GetHeight()
	)

	return L2/L1
end


-- Update the panel to match the guide frame's location (but don't save to database until we're finished)
local function GetGuideFramePosition(guideObject)
	local x, y = guideObject.frame:GetCenter()
	x = (x - (UIParent:GetWidth()  * UIParent:GetEffectiveScale()) / 2) / UIParent:GetEffectiveScale()
	y = (y - (UIParent:GetHeight() * UIParent:GetEffectiveScale()) / 2) / UIParent:GetEffectiveScale()
	
	return x, y
end


-- Update rotation according to configMode and slider position
function LibAvionEmbed:SliderListener(animationID)
	local guideObject = self.vars.linkCache[animationID].guideObject
	
	-- In start-value mode
	if (guideObject.configMode == 0) then
		self.vars.animationList[animationID].rotation = guideObject.slider:GetValue() / 180 * math.pi

	--In animation-value mode
	elseif (guideObject.configMode == 1) then
		self.vars.animationList[animationID].animation.rotation = guideObject.slider:GetValue() / 180 * math.pi
	end
	
	-- Update animation
	self:SetTestFrameStatus(animationID, nil)
end


-- Update postition of frame, based on position of guideFrame
function LibAvionEmbed:MoveListener(animationID)
	local guideObject = self.vars.linkCache[animationID].guideObject
	
	-- Update the panel to match the guide frame's location (but don't save to database until we're finished)
	local x, y = GetGuideFramePosition(guideObject)

	-- If not in animation mode, update size(w&h) (and position)
	if (guideObject.configMode == 0) then
		self.vars.animationList[animationID].position = {
			x = x,
			y = y
		}
	elseif (guideObject.configMode == 1) then
		if (self.vars.animationList[animationID].isAnimated.position) then
			self.vars.animationList[animationID].animation.position = {
				x = x,
				y = y
			}
		end
	end
	
	-- Update animation
	self:SetTestFrameStatus(animationID, nil)
end


-- Update size of frame, based on size of guideFrame
function LibAvionEmbed:ResizeListener(animationID)
	local guideObject = self.vars.linkCache[animationID].guideObject
	local animationTable = self.vars.animationList[animationID]
--	local x, y = GetGuideFramePosition(guideObject)
	
	-- Update size (w&h)
	if (guideObject.configMode == 0) then
		guideObject.frame:ClearAllPoints()
		guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", animationTable.position.x*UIParent:GetEffectiveScale(), animationTable.position.y*UIParent:GetEffectiveScale())
		
		self.vars.animationList[animationID].size = {
			width  = guideObject.frame:GetWidth()  * 1.2,
			height = guideObject.frame:GetHeight() * 1.2
		}
		
	-- Update scale
	elseif (guideObject.configMode == 1) then
		guideObject.frame:ClearAllPoints()
		guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", animationTable.animation.position.x*UIParent:GetEffectiveScale(), animationTable.animation.position.y*UIParent:GetEffectiveScale())
		
		local scale = GetGuideFrameScale(guideObject, self.vars.animationList[animationID])
		
		guideObject.frame:SetWidth(scale  * self.vars.animationList[animationID].size.width  / 1.2)
		guideObject.frame:SetHeight(scale * self.vars.animationList[animationID].size.height / 1.2)
		
		if (self.vars.animationList[animationID].isAnimated.scale) then
			self.vars.animationList[animationID].animation.scale = scale
		end
	end
	
	-- Update animation
	self:SetTestFrameStatus(animationID, nil)
end


-- This is called each time mouse button is pressed (and hold) 
function LibAvionEmbed:MouseDownListener(arg1, animationID)
	local animationObject 	= self.vars.linkCache[animationID].animationObject
	local guideObject 		= self.vars.linkCache[animationID].guideObject
	local animationTable 	= self.vars.animationList[animationID]
		
	-- Right-click mean, change config mode between start and animate values
	if (arg1 == "RightButton" or arg1 == true) then
		-- Switch config mode
		if (arg1 ~= true) then
			guideObject.configMode = (guideObject.configMode + 1) % 2
		end
		
		-- Currently in start-value mode
		if (guideObject.configMode == 0) then
			-- Update background color
			guideObject.texture:SetTexture(0, 1, 1)
			guideObject.slider:SetBackdropColor(0, 1, 1, 1)
			guideObject.slider:SetBackdropBorderColor(0, 1, 1, 1)
			guideObject.text:SetText(animationTable.name .. "\n" .. L["GuideFrameMode0"])
			
			-- Reset position
			guideObject.frame:ClearAllPoints()
			guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", animationTable.position.x*UIParent:GetEffectiveScale(), animationTable.position.y*UIParent:GetEffectiveScale())
			guideObject.frame:SetWidth(animationTable.size.width   / 1.2)
			guideObject.frame:SetHeight(animationTable.size.height / 1.2)
			guideObject.slider:SetValue(animationTable.rotation / math.pi * 180)
			
			-- Set animation to its beginning
			animationObject.elapsedTimes = 0
		
		-- Currently in animate value mode
		elseif (guideObject.configMode == 1) then
			-- Update background color
			guideObject.texture:SetTexture(1, 1, 0)
			guideObject.slider:SetBackdropColor(1, 1, 0, 1)
			guideObject.slider:SetBackdropBorderColor(1, 1, 0, 1)
			guideObject.text:SetText(animationTable.name .. "\n" .. L["GuideFrameMode1"])
			
			-- Reset position
			guideObject.frame:ClearAllPoints()
			if (animationTable.isAnimated.position) then
				guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", animationTable.animation.position.x*UIParent:GetEffectiveScale(), animationTable.animation.position.y*UIParent:GetEffectiveScale())
			else
				guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", animationTable.position.x*UIParent:GetEffectiveScale(), animationTable.position.y*UIParent:GetEffectiveScale())
			end
			if (animationTable.isAnimated.scale) then
				guideObject.frame:SetWidth(animationTable.animation.scale  * animationTable.size.width  / 1.2)
				guideObject.frame:SetHeight(animationTable.animation.scale * animationTable.size.height / 1.2)
			else
				guideObject.frame:SetWidth(animationTable.size.width   / 1.2)
				guideObject.frame:SetHeight(animationTable.size.height / 1.2)
			end
			guideObject.slider:SetValue(animationTable.animation.rotation / math.pi * 180)
			
			-- Set animation to its end
			animationObject.elapsedTimes = animationTable.animationTime / LibAvion.vars.updateInterval
		end
		
		-- Adjust animation frame
		self:SetTestFrameStatus(animationID, nil)

		
	-- On left-clicks, listen for mouse-dragging
	elseif (arg1 == "LeftButton") then
		-- Set vars to figure out if the mouse is in the resize area or not
		local screenX, screenY = GetCursorPosition()
		local panelX = guideObject.frame:GetRight()
		local panelY = guideObject.frame:GetBottom()

		-- Adjust for screen scale
		local scale = guideObject.frame:GetEffectiveScale()
		panelX = panelX * scale
		panelY = panelY * scale
		
		-- Set bounds for lower-right hand corner (inside resize texture)
		local check1 = screenX <= panelX + 14
		local check2 = screenX >= panelX - 14
		local check3 = screenY <= panelY + 14
		local check4 = screenY >= panelY - 14
		
		-- Start resizing
		if check1 and check2 and check3 and check4 then
			if (guideObject.configMode == 0 or animationTable.isAnimated.scale) then
				-- Attach an OnUpdate call which fires every repaint while we're resizing
				guideObject.frame:SetScript("OnUpdate", function() self:ResizeListener(animationID) end)
			
				-- System call
				guideObject.frame:StartSizing("BOTTOMRIGHT")
			end
			
		-- Start moving
		else
			if (guideObject.configMode == 0 or animationTable.isAnimated.position) then
				-- Attach an OnUpdate call which fires every repaint while we're moving
				guideObject.frame:SetScript("OnUpdate", function() self:MoveListener(animationID) end)
			
				-- System call
				guideObject.frame:StartMoving()
			end
		end
	end
end


-- Mouse has been released, set and save settings
function LibAvionEmbed:MouseUpListener(animationID)
	-- Stop moving or resizing
	 self.vars.linkCache[animationID].guideObject.frame:StopMovingOrSizing()
	
	-- Remove the move/resize listeners
	 self.vars.linkCache[animationID].guideObject.frame:SetScript("OnUpdate", nil)

	-- Update position once more
	self:MoveListener(animationID)
	self:ResizeListener(animationID)
	
	-- Update Options menu
	self:GetAvionOptionTable()
end


-------------------------------------------------------------------------------
-- Library - Test Frame
-------------------------------------------------------------------------------
function LibAvionEmbed:ShowTestFrame(animationID)
	if (not self:IsTestFrameShown(animationID)) then
		-- Save link to animation und guide frame that's beeing used
		self.vars.linkCache[animationID] = {}
		self.vars.linkCache[animationID].animationObject, _	= GetAvailableFrame()
		self.vars.linkCache[animationID].guideObject 		= MakeGuideFrame()
		
		-- Convert to intern format
		local animationTable 	= AnimationInternToExtern(self.vars.animationList[animationID])
		local animationObject 	= self.vars.linkCache[animationID].animationObject
		local guideObject 		= self.vars.linkCache[animationID].guideObject
		
		-- Save which triggerID triggerd this animation
		animationObject.triggerID = animationID
		
		-- Update Text & position
		guideObject.frame:Show()
		self:MouseDownListener(true, animationID)
		
		-- Show frame
		InitFrame(animationObject, animationTable)
		
		-- Init frame methods
		guideObject.slider:SetScript(		"OnValueChanged",	function() self:SliderListener(animationID) 	end)
--		guideObject.menuButton:SetScript(	"OnClick", 			function() --[[ <!!!> ]] end)
--		guideObject.mirrorX:SetScript(		"OnClick", 			function() --[[ <!!!> ]] end)
--		guideObject.mirrorY:SetScript(		"OnClick", 			function() --[[ <!!!> ]] end)
--		guideObject.closeButton:SetScript(	"OnClick", 			function() --[[ <!!!> ]] end)
		guideObject.frame:SetScript(		"OnMouseDown",		function() self:MouseDownListener(arg1, animationID) 	end)
		guideObject.frame:SetScript(		"OnMouseUp",		function() self:MouseUpListener(animationID) 	end)
	end
end


-- Hide test frame (eg. if seting up options)
function LibAvionEmbed:HideTestFrame(animationID)
	-- Create frame-memory
	if (self:IsTestFrameShown(animationID)) then
		local animationObject 	= self.vars.linkCache[animationID].animationObject
		local guideObject 		= self.vars.linkCache[animationID].guideObject
		
		-- Hide frame and its not inUse anymore...
		animationObject.texture:SetTexture("Interface\\Icons\\Temp.blp")
		animationObject.frame:Hide()
		animationObject.inUse = false
		
		-- Clean up
		guideObject.text:SetText("")
		guideObject.frame:Hide()
		
		-- Stop script and clear mem (or atleast give doggy GC something to eat)
		guideObject.slider:SetScript(		"OnValueChanged",	nil)
		guideObject.menuButton:SetScript(	"OnClick", 			nil)
		guideObject.mirrorX:SetScript(		"OnClick", 			nil)
		guideObject.mirrorY:SetScript(		"OnClick", 			nil)
		guideObject.closeButton:SetScript(	"OnClick", 			nil)
		guideObject.frame:SetScript(		"OnMouseDown",		nil)
		guideObject.frame:SetScript(		"OnMouseUp",		nil)
		
		-- Feed frame to garbage collection
		guideObject.frame 			= nil
		guideObject.texture 		= nil
		guideObject.text 			= nil
		guideObject.resizeTexture	= nil
		guideObject.slider 			= nil
		guideObject.menuButton 		= nil
		guideObject.mirrorX 		= nil
		guideObject.mirrorY 		= nil
		guideObject.closeButton 	= nil
		guideObject.configMode		= nil
		guideObject 				= nil
		
		-- Delete frame from animation cache
		self.vars.linkCache[animationID] = nil
	end
end


-- Is a frame shown
function LibAvionEmbed:IsTestFrameShown(animationID)
	if (self.vars.linkCache[animationID]) then
		return true
	else
		return false
	end
end


-- Set animation-status of an animation to a certain time
function LibAvionEmbed:SetTestFrameStatus(animationID, timer)
	-- Make sure frame is shown
	if (self:IsTestFrameShown(animationID)) then
		local animationObject = self.vars.linkCache[animationID].animationObject
		
		-- Keep current timer, just update animation
		if (timer == nil) then
			timer = animationObject.elapsedTimes * LibAvion.vars.updateInterval
		end
	
		-- Prepare table for Avion intern usage
		local animationTable = AnimationInternToExtern(self.vars.animationList[animationID])
		
		-- Reset Frame
		InitFrame(animationObject, animationTable)

		-- Set timer
		animationObject.elapsedTimes = timer / LibAvion.vars.updateInterval
		
		-- Update animation
		UpdateAnimation(animationObject, animationTable, true, false)
		
		-- Reset timer
		animationObject.elapsedTimes = timer / LibAvion.vars.updateInterval
	end
end


-- Get animation-status of an animation to a certain time
function LibAvionEmbed:GetTestFrameStatus(animationID)
	-- Make sure frame is shown
	if (self:IsTestFrameShown(animationID)) then
		return self.vars.linkCache[animationID].animationObject.elapsedTimes * LibAvion.vars.updateInterval
	else
		return 0
	end
end


-------------------------------------------------------------------------------
-- Library - Re-Emed
-------------------------------------------------------------------------------
for target, _ in pairs(LibAvionEmbed.mixinTargets) do
	LibAvionEmbed:Embed(target)
end


-------------------------------------------------------------------------------
-- Dokumentation - Variables
-------------------------------------------------------------------------------
--[[
VAR: AnimationTable
TYPE: Table
DESC: It cantains all needed information to start an animation:
	{
		alpha 		= Alpha value when its shown (number)
		color 			= {
			r		= Red color when its shown (number)
			g 		= Green color when its shown (number)
			b 		= Blue color when its shown (number)
		}
		position 		= {
			x 		= X-Position on screen when its shown (number)
			y 		= Y-Position on screen when its shown (number)
		}
		size 			= {
			width 	= Width when its shown (number)
			height 	= Height when its shown (number)
		}
		rotation 		= Rotation of texture when its shown (number)
		isAnimated 		= {
			color 		= Should color be animated? (bool)
			position 	= Should position be animated? (bool)
			scale 		= Should scale be animated? (bool)
			rotation 	= Should rotation be animated? (bool)
		}
		animation 		= {
			color 		= {
				r 	= Red color that is beeing animated to (number)
				g 	= Greem color that is beeing animated to (number)
				b 	= Blue color that is beeing animated to (number)
			}, 
			position 	= {
				x 	= X-Position that is beeing animated to (number)
				y 	= Y-Position that is beeing animated to (number)
			}, 
			scale 		= Scale that is beeing animated to (number)
			rotation 	= Rotations of texture that is beeing animated to (number)
		}
		animationFunc	= {
			alpha 	= Smoothness/speed of alpha animation (function)
			color 		= Smoothness/speed of color animation (function)
			position 	= Smoothness/speed of position animation (function)
			scale 		= Smoothness/speed of scale animation (function)
			rotation 	= Smoothness/speed of rotation animation (function)
		}
NOTE: All of this function HAVE to be pointers to function with head 
"function (param, rnd)", where param gonna be a value between 0 (start)
and 1 (end), depending on the time the animation is still running.
rnd is the value that can be set in the options dialogue under "Randomness",
its meant to add some random effect.
		addRandom 		= {
			alpha 	= Add some randomness to alpha animation (number)
			color 		= Add some randomness to color animation (number)
			position 	= Add some randomness to position animation (number)
			scale 		= Add some randomness to scale animation (number)
			rotation 	= Add some randomness to rotation animation (number)
		}
		animateAndFade 	= Continue animation when fading out 
NOTE: This will NOT work when Fade-Stopping the animation! (bool)
		animationTime 	= The time the "animation" phase should take
		fadeTime 		= The time it should take the frame to fade 
					out after completion (number)
		waitTime 		= The time it should take from triggering the 
					animation until the actual animation should start 
NOTE: Can be used to create difficult multi-animations. (number)
		showWhileWait 	= Allready show the frame/animation while
					 "waitTime" is yet running (bool)
		textureFile 		= Texture file (file link, no SharedMedia!) to 
					the texture that should be used (string)
		soundFile 		= Sound file (file link, no SharedMedia!) to 
					the sound that is triggered as soon as the 
					animation starts (string)
		mirror 		= {
			x 		= Mirror texture on x-axis (bool)
			y 		= Mirror texture on y-axis (bool)
		}
		noEnd 		= The animation won't fade when its finished, but
					will rather need to be stopped with 
					LibAvion:StopAnimation(...) (bool)
}


VAR: LibAvion.vars 
TYPE: Table
DESC: Contains all variables "shared" among LibAvions functions, including variables
	.animationCache
	.updateInterval
	.frameCache
	.frameCounter
	.animationTableCache
	.funcCache
	.media
For more, see below.


VAR: LibAvion.vars.animationCache
TYPE: Table
DESC: Stores all currently active (running) animations, more speficly the unique key of the
frame (in LibAvion.vars.frameCache), mostly refered to as the FrameID that was assossiated to 
this animation. The frame thats beeing animated.
This table is accessed by the unique key, that was used to start the animation, namely triggerID.


VAR: LibAvion.vars.updateInterval
TYPE: Number
DESC: Update rate for all animations, eg. 0.05 means roughly 25 updates per second.


VAR: LibAvion.vars.frameCache 
Type: Table
DESC: Table that contains a pool of created/available frame objects to be used for visualising
an animation. It has "LibAvion.vars.frameCounter" number of entrys and the unique key that is
used to access it is called frameID.
Each entry inhabits the following entrys:
	.inUse		- Say if the frame is currently beeing used for visualising an animation
	.frame		- the actuell fram object that can be shown or hidden
	.texture		- the texture object
	.elapsedTimes	- iterater that (if the animation is currently running) tells how often it has been updated allready


VAR: LibAvion.vars.frameCounter
TYPE: Number
DESC: Its just #LibAvion.vars.frameCache, nothing more, nothing less


VAR: LibAvion.vars.animationTableCache
TYPE: Table
DESC: Temporary (as long as the animation frame is shown/still active) saves each animations
AnimationTable. It is accessed by the unique key triggerID.


VAR: LibAvion.vars.funcCache
TYPE: Table
DESC: After calling LibAvionEmbed.LinkAnimationDB this little table will hold pointers to each
function, that is created using entrys from LibAvion.vars.media.func (see below) as bodys.
Its used to quickly access this functions when running an animation, so they won't have to be created
and garbaged each time an animation is ran or stopped. This way they are kept permenently (but only
ONCE) in the memory. This table is accessed by the name of the function see below).


VAR: LibAvion.vars.media
TYPE: Table
DESC: Kind of a replacement for SharedMedia. (Not that I don't like it! Its just thus LibAvion is 
mostly lib and framework independent and addon dev can still use SharedMedia to add textures and
sounds by using LibAvion:AddTexture, LibAvionAddSound, LibAvion:AddFunction but also now can
custom texture beening added without the use of SharedMedia)


VAR: LibAvion.vars.media.func
TYPE: Table
DESC: Holds functions-bodys, format is as follows:
	["NAME"] = "BODY"
and gets pasted into:
	return function (param, rnd) 
		if (rnd) then
			return (<INSERT BODY> + math.random(-rnd, rnd) / 10000) 
		else 
			return (<INSERT BODY>)
		end
	end
New functions can be added with LibAvion:AddFunction(NAME, BODY).


VAR: LibAvion.vars.media.texture
TYPE: Table
DESC: Holds possible textures, syntax is as follows
	["NAME"] = "LOCATION OF FILE"
New textures can be added with LibAvion:AddTexture(NAME, FILE).


VAR: LibAvion.vars.media.sound 
TYPE: Table
DESC: Holds possible sound, syntax is as follows
	["NAME"] = "LOCATION OF FILE"
New sounds can be added with LibAvion:AddSound(NAME, FILE).


VAR: LibAvionEmbed.vars 
TYPE: Table
DESC: Contains all variables that are unique to each instance of LibAvionEmbed, containing the following entrys:
	.linkCache
	.animationList
For more, see below.


VAR: LibAvionEmbed.vars.linkCache
TYPE: Table
DESC: Much like LibAvion.vars.animationCache (see above) this contains the frame object that is accossiated
a currently "beeing tested" animation. It is accessed by the unique key animationID since it assossiates a
"beeing tested" animation with its currently shown frame object.


VAR: LibAvionEmbed.vars.animationList
TYPE: Table
DESC: Contains a list of animationTables (see beginning of Doc), its entry are accessed 
by the unique key animationID.
	
	
NOTE: 
Since LibAvion is a sole library, variables starting with LibAvion. are the same for alle "instances" of LibAvion and LibAvionEmbed,
while LibAvionEmbed is made to be embedded all variables starting with LibAvionEmbed. are unique to each addon/library embedding
LibAvionEmbed! (Just a note to myself ^.^)

EOF]]

--DEFAULT_CHAT_FRAME:AddMessage("1: " .. x .. " | " .. y)